program cntcart2;

uses crt,dos,graph,grutils,mouse;

type objecttype = record x, y, fx, fy, vx, vy, mass: double end;
     csystype = record p,r,e,o:double; end;

 var bob,cart: objecttype;
     cartpos, cartvel, bobpos, bobvel, bobaccel: csystype;
     deltax, deltay, dt, impulse: double;
     l,l0,f,g,ke,r0: double;
     iters: longint;
     time,level: integer;
     control: boolean;
     ch: char;


procedure initpendulum;
begin
 dt := 1e-4;   { sec }
 ke := 1e7;    { newton/meter}
 l0 := 1.0;    { meter}
 g  := 9.8;    { meter/sec^2}
 iters := 0;
 control := true;

 with bob do
 begin
  vx := 0.0;  {m/sec}
  vy := 0.0;
  x := l0*sin(0);
  y := l0*cos(0);  {meter}
  mass := 0.5; {kg}
 end;

 with cart do
 begin
  vx := 0.0;
  vy := 0.0;
  x := 0.0;
  y := 0.0;
  mass := 0.5; {kg}
 end;
end;



procedure showpendulum;
const oldx: integer = 0;
      oldy: integer = 0;
      oldcx: integer = 0;
      oldcy: integer = 0;
begin
 if iters > 0 then
 begin
  setcolor(black);
  rectangle(hcenter - 30 + oldcx,
            vcenter,hcenter + 30 + oldcx,vcenter- 15);
  circle(hcenter + oldx, vcenter - oldy,10);
  circle(hcenter + oldcx - 20, vcenter - oldcy,7);
  circle(hcenter + oldcx + 20, vcenter - oldcy,7);
  line(hcenter + oldcx,vcenter - oldcy,
       hcenter + oldx,vcenter - oldy);
 end;

 oldx := round(200*bob.x);
 oldy := round(200*bob.y);
 oldcx := round(200*cart.x);
 oldcy := round(200*cart.y);

 setcolor(white);
 rectangle(hcenter - 30 + oldcx,
            vcenter,hcenter + 30 + oldcx,vcenter- 15);
 circle(hcenter + oldx, vcenter - oldy,10);
 circle(hcenter + oldcx - 20, vcenter - oldcy,7);
 circle(hcenter + oldcx + 20, vcenter - oldcy,7);
 line(hcenter + oldcx - 20,vcenter + 7,hcenter + oldcx + 20,vcenter+ 7);
 line(hcenter + oldcx,vcenter - oldcy,
      hcenter + oldx,vcenter - oldy);
 line(0, vcenter + 7,hsize,vcenter + 7);

end;

procedure showref;
const oldref: integer = 0;
begin
 setcolor(black);
 line(oldref,25,oldref,35);
 line(oldref-1,25,oldref-1,35);
 oldref := hcenter + round(mousex*0.2);
 setcolor(lightred);
 line(oldref,25,oldref,35);
 line(oldref-1,25,oldref-1,35);
 setcolor(white);
end;

procedure dynamics;
begin
  with bob do
  begin
   deltax := bob.x - cart.x;
   deltay := bob.y - cart.y;
   l := sqrt(sqr(deltax) + sqr(deltay));
   f := (l0 - l)*ke;
   fx := f*deltax/l + 0.001*vx + impulse;
   fy := f*deltay/l - mass*g + 0.001*vy;
   vx := vx + fx/mass* dt;
   x := x + (vx + 0.5*fx*dt/mass)*dt;
   vy := vy + fy/mass* dt;
   y := y + (vy + 0.5*fy*dt/mass)*dt;
  end;
  with cart do
  begin
   if control then fx := -bob.fx + cartvel.o
   else fx := -bob.fx;
   vx := vx + fx/mass*dt;
   x := x + (vx + 0.5*fx/mass*dt)*dt;
  end;
end;



procedure cartvelcont;
begin
 with cartvel do
 begin
  p := cart.vx;
  r := cartpos.o;
  e := r - p;
  o := 200.0* e;
 end;
end;

procedure cartposcont;
begin
 with cartpos do
 begin
  p := cart.x;
  r := bobaccel.o;
  e := r - p;
  o := 200.0*e;
 end
end;

procedure bobaccelcont;
begin
 with bobaccel do
 begin
  p := bob.x - cart.x;
  if level <> 1 then
  r := bobvel.o
  else r := mousex*0.0001;
  e := r - p;
  o := -200*e;
 end
end;

procedure bobvelcont;
begin
 with bobvel do
 begin
  p := bob.vx;
  if level <> 2 then
  r := bobpos.o
  else r := mousex*0.001;
  e := r - p;
  o := o + 0.001*(0.5*e - o);
  if o > 0.4 then o := 0.4
  else
  if o < -0.4 then o := -0.4;
 end;
end;

procedure bobposcont;
begin
 with bobpos do
 begin
  r := mousex*0.001;
  p := bob.x;
  e := r - p;
  o := 1.5*e;
 { if o > 0.7 then o := 0.7
  else
  if o < -0.7 then o := -0.7;}
 end;
end;


procedure showlegend;
begin
 setcolor(white);
 outtextxy(0,0,'PRESS "q" TO QUIT');
 outtextxy(0,10,'PRESS "z" TO TURN OFF CONTROL');
 outtextxy(0,vsize - 24,'REF POS');
 setcolor(lightcyan);
 outtextxy(0,vsize - 37,'BOB POS');
 setcolor(lightred);
 outtextxy(0,vsize - 50,'CART POS');
 setcolor(white);
 outtextxy(100,vsize-50,'impulse = i');
 outtextxy(100,vsize-37,'force on/off = f');
 outtextxy(300,0,'USE MOUSE TO SET REF POSITION OF BOB');
 ch := chr(0);
end;

procedure checkcommand;
const x: integer = 0;
      y: integer = 0;
begin
  ch := ' ';
  if keypressed then ch := readkey;
  case ch of
   'z': control := not control;
   'i': begin
         impulse := 2.0;
         time := 1000;
        end;
   'f': if impulse = 0.0 then impulse := 1.0 else impulse := 0.0;
   '1': begin level := 1; mousex := 0; end;
   '2': begin level := 2; mousex := 0; end;
   '3': begin level := 3; mousex := 0; end;
  end;
  if ch <> 'q' then ch := ' ';
  if time > 0 then dec(time);
  if time = 1 then impulse := 0.0;
  if time > 800 then
   begin
    setcolor(lightred);
    x := hcenter + round(200*bob.x);
    y := vcenter -200*round( bob.y);
    line(x-50,y,x-10,y);
   end;
  if time = 800 then
  begin
   setcolor(black);
   line(x-50,y,x-10,y);
 end;
end;

begin
 initgraphics;
 initpendulum;
 if not initmouse then exit;
 l := l0;      {L-nought, not 10.0}
 showlegend;
 showpendulum;
 level := 5;
 ch := ' ';
 while ch <>'q' do
 begin
  dynamics;
  if control then
  begin
   bobposcont;
   bobvelcont;
   bobaccelcont;
   cartposcont;
   cartvelcont;
  end;
  if (iters mod 100) = 0 then
  begin
   showpendulum;
   showref;
   if iters < 320000 then
   begin
    putpixel(iters div 500, vcenter + 120 - round(70*bobpos.r),white);
    putpixel(iters div 500, vcenter + 120 - round(70*cart.x),lightred);
    putpixel(iters div 500, vcenter + 120 - round(70*bob.x),lightcyanq);
    putpixel(iters div 500, vcenter + 120,white);
   end
   else if iters = 320000 then
    begin
     clearviewport;
     iters := 0;
     showlegend;
    end;
   delay(5);
   readmouse;
  end;
  inc(iters);
  checkcommand;
 end;
 closegraph;
end.

